#include <bits/stdc++.h>
using namespace std;

#define SIZE(x) int((x).size())

int main() {
    int R, C, M, clickR, clickC;
    cin >> R >> C >> M >> clickR >> clickC;

    map< vector< vector<int> >, vector< vector<int> > > OBSERVATIONS;

    vector< pair<int,int> > not_adjacent_to_click;
    for (int r=0; r<R; ++r) for (int c=0; c<C; ++c) if (max( abs(r-clickR) , abs(c-clickC) ) >= 2) not_adjacent_to_click.push_back( {r,c} );
    if (SIZE(not_adjacent_to_click) <= M) return 0;

    vector<int> is_mine(M,1);
    is_mine.resize( SIZE(not_adjacent_to_click), 0 );
    sort( is_mine.begin(), is_mine.end() );

    // generate all possible distributions of M mines onto the cells that are in distance 2+ from the click
    // categorize them according to what the player would see
    do {
        // place the mines onto a 2D plan
        vector< vector<int> > plan(R, vector<int>(C,0));
        for (int i=0; i<SIZE(not_adjacent_to_click); ++i) if (is_mine[i]) {
            int mr, mc;
            tie(mr,mc) = not_adjacent_to_click[i];
            plan[mr][mc] = 1;
        }

        // compute the annotation numbers
        vector< vector<int> > numbers(R, vector<int>(C,0));
        for (int r=0; r<R; ++r) for (int c=0; c<C; ++c) for (int dr=-1; dr<=1; ++dr) for (int dc=-1; dc<=1; ++dc) {
            int cr=r+dr, cc=c+dc;
            if (!(0<=cr && cr<R)) continue;
            if (!(0<=cc && cc<C)) continue;
            numbers[r][c] += plan[cr][cc];
        }

        // run a BFS to find all visible cells
        vector< vector<int> > visible(R, vector<int>(C,-1));
        visible[clickR][clickC] = 0;
        queue<int> Q;
        Q.push(clickR);
        Q.push(clickC);
        while (!Q.empty()) {
            int curr = Q.front(); Q.pop();
            int curc = Q.front(); Q.pop();
            for (int dr=-1; dr<=1; ++dr) for (int dc=-1; dc<=1; ++dc) {
                int nr=curr+dr, nc=curc+dc;
                if (!(0<=nr && nr<R)) continue;
                if (!(0<=nc && nc<C)) continue;
                if (visible[nr][nc] >= 0) continue;
                visible[nr][nc] = numbers[nr][nc];
                if (numbers[nr][nc] == 0) {
                    Q.push(nr);
                    Q.push(nc);
                }
            }
        }
        
        OBSERVATIONS[ visible ].push_back(is_mine);
    } while (next_permutation( is_mine.begin(), is_mine.end() ));
   
    // for each "equivalence class" in OBSERVATIONS, check whether it is a solution
    for (const auto &possibility : OBSERVATIONS) {
        // go over all mine placements in this equivalence class
        // for each cell, count the number of times it has a mine
        vector< vector<int> > times_mine(R, vector<int>(C,0));
        for (const auto &placement : possibility.second) {
            for (int i=0; i<SIZE(not_adjacent_to_click); ++i) if (placement[i]) {
                int mr, mc;
                tie(mr,mc) = not_adjacent_to_click[i];
                ++times_mine[mr][mc];
            }
        }

        // check whether all not-visible cells contain a mine at least once but not always
        bool good = true;
        for (int i=0; i<SIZE(not_adjacent_to_click); ++i) {
            int mr, mc;
            tie(mr,mc) = not_adjacent_to_click[i];
            if (possibility.first[mr][mc] >= 0) continue; // skip visible cells
            good &= ( times_mine[mr][mc] > 0 && times_mine[mr][mc] < SIZE(possibility.second) );
        }

        // this is a valid solution, but we don't just want the whole equivalence class
        // instead, we want to select its small-enough subset that still has the desired property
        // this is just a quick and dirty heuristic: 
        // for each cell we keep track whether we already have a configuration when it does/doesn't contain a mine
        // we take a configuration of mines iff it provides some new value
        
        if (good) {

            vector< vector<int> > WANTED;
            {
                vector< vector<bool> > got_space(R, vector<bool>(C,false));
                vector< vector<bool> > got_mine(R, vector<bool>(C,false));
                for (const auto &placement : possibility.second) {
                    bool useful = false;
                    for (int i=0; i<SIZE(not_adjacent_to_click); ++i) {
                        int mr, mc;
                        tie(mr,mc) = not_adjacent_to_click[i];
                        if (possibility.first[mr][mc] >= 0) continue; // skip visible cells
                        if ( placement[i] && !got_mine[mr][mc]) { got_mine[mr][mc] = true; useful = true; }
                        if (!placement[i] && !got_space[mr][mc]) { got_space[mr][mc] = true; useful = true; }
                    }
                    if (useful) WANTED.push_back(placement);
                }
            }
            // check that our heuristic produced a set that is small enough
            assert( SIZE(WANTED) <= R*C );
            
            // print the configurations as a single line of C++ code
            printf("PRECOMPUTED[ make_tuple(%d,%d,%d,%d,%d) ] = {",R,C,M,clickR,clickC);
            for (const auto &placement : WANTED) {
                vector< vector<bool> > mine(R,vector<bool>(C,false));
                for (int i=0; i<SIZE(not_adjacent_to_click); ++i) if (placement[i]) {
                    int mr, mc;
                    tie(mr,mc) = not_adjacent_to_click[i];
                    mine[mr][mc] = true;
                }
                cout << "{";
                for (int r=0; r<R; ++r) { cout << "\""; for (int c=0; c<C; ++c) cout << (mine[r][c] ? '*' : '.'); cout << "\", "; }
                cout << "}, ";
            }
            cout << "};\n";
            return 0;
        }
    }
}
